@@ -1,6 +1,9 @@
 
 var deprecate = require('../..')('my-lib')
 
+exports.fn = fn
+exports.prop = 'thingie'
+
 exports.old = function () {
   deprecate('old')
 }
@@ -59,6 +62,21 @@ exports.automsganon = function () {
   (function () { deprecate() }())
 }
 
+exports.fnprop = function thefn() {}
+exports.fnprop.propa = 'thingie'
+exports.fnprop.propautomsg = 'thingie'
+
+deprecate.property(exports.fnprop, 'propa', 'fn propa gone')
+deprecate.property(exports.fnprop, 'propautomsg')
+
+exports.layerfn = function () {
+  exports.oldfn()
+}
+
+exports.layerprop = function () {
+  exports.propa
+}
+
 function fn(a1, a2) {
   return a2
 }
