@@ -1,8 +1,12 @@
 
 var basename = require('path').basename
+var bufferConcat = require('../lib/compat').bufferConcat
 var depd = require('..')
 var mylib = require('./fixtures/my-lib')
+var path = require('path')
+var script = path.join(__dirname, 'fixtures', 'script.js')
 var should = require('should')
+var spawn = require('child_process').spawn
 
 describe('depd(namespace)', function () {
   it('creates deprecated function', function () {
@@ -37,6 +41,19 @@ describe('deprecate(message)', function () {
     stderr.should.match(/\.js:[0-9]+:[0-9]+/)
   })
 
+  it('should log call site regardless of Error.stackTraceLimit', function () {
+    function callold() { mylib.old() }
+    var limit = Error.stackTraceLimit
+    try {
+      Error.stackTraceLimit = 1
+      var stderr = captureStderr(callold)
+      stderr.should.containEql(basename(__filename))
+      stderr.should.match(/\.js:[0-9]+:[0-9]+/)
+    } finally {
+      Error.stackTraceLimit = limit
+    }
+  })
+
   it('should log call site within eval', function () {
     function callold() { eval('mylib.old()') }
     var stderr = captureStderr(callold)
@@ -219,6 +236,13 @@ describe('deprecate.function(fn, message)', function () {
     ret.should.equal(2)
   })
 
+  it('should show call site outside scope', function () {
+    function callold() { mylib.layerfn() }
+    var stderr = captureStderr(callold)
+    stderr.should.containEql(' oldfn ')
+    stderr.should.match(/test.js:[0-9]+:[0-9]+/)
+  })
+
   it('should only warn once per call site', function () {
     function callold() {
       for (var i = 0; i < 5; i++) {
@@ -232,6 +256,17 @@ describe('deprecate.function(fn, message)', function () {
     stderr.split('invoke').should.have.length(6)
   })
 
+  it('should handle rapid calling of deprecated thing', function () {
+    function callold() {
+      for (var i = 0; i < 10000; i++) {
+        mylib.oldfn()
+      }
+    }
+
+    var stderr = captureStderr(callold)
+    stderr.split('deprecated').should.have.length(2)
+  })
+
   it('should warn for different calls on same line', function () {
     function callold() {
       mylib.oldfn(), mylib.oldfn()
@@ -327,6 +362,29 @@ describe('deprecate.property(obj, prop, message)', function () {
     stderr.split(fileline[0]).should.have.length(3)
   })
 
+  it('should show call site outside scope', function () {
+    function callold() { mylib.layerprop() }
+    var stderr = captureStderr(callold)
+    stderr.should.containEql(' propa ')
+    stderr.should.match(/test.js:[0-9]+:[0-9]+/)
+  })
+
+  describe('when obj is a function', function () {
+    it('should log on access to property on function', function () {
+      function callprop() { mylib.fnprop.propa }
+      var stderr = captureStderr(callprop)
+      stderr.should.containEql(' deprecated ')
+      stderr.should.containEql(' fn propa gone ')
+    })
+
+    it('should generate message on named function', function () {
+      function callprop() { mylib.fnprop.propautomsg }
+      var stderr = captureStderr(callprop)
+      stderr.should.containEql(' deprecated ')
+      stderr.should.containEql(' thefn.propautomsg ')
+    })
+  })
+
   describe('when value descriptor', function () {
     it('should log on access and set', function () {
       function callold() { mylib.propa }
@@ -440,6 +498,14 @@ describe('process.env.NO_DEPRECATION', function () {
   var error
   function ondeprecation(err) { error = err }
 
+  beforeEach(function () {
+    error = null
+  })
+
+  afterEach(function () {
+    process.removeListener('deprecation', ondeprecation)
+  })
+
   after(function () {
     process.env.NO_DEPRECATION = ''
   })
@@ -464,6 +530,14 @@ describe('process.env.NO_DEPRECATION', function () {
     captureStderr(newlib.old).should.be.empty
   })
 
+  it('should emit "deprecation" events anyway', function () {
+    process.env.NO_DEPRECATION = 'thing-lib'
+    var thinglib = require('./fixtures/thing-lib')
+    process.on('deprecation', ondeprecation)
+    captureStderr(thinglib.old).should.be.empty
+    error.namespace.should.equal('thing-lib')
+  })
+
   describe('when *', function () {
     it('should suppress any namespace', function () {
       process.env.NO_DEPRECATION = '*'
@@ -474,6 +548,112 @@ describe('process.env.NO_DEPRECATION', function () {
   })
 })
 
+describe('process.env.TRACE_DEPRECATION', function () {
+  before(function () {
+    process.env.TRACE_DEPRECATION = 'trace-lib'
+  })
+
+  after(function () {
+    process.env.TRACE_DEPRECATION = ''
+  })
+
+  it('should trace given namespace', function () {
+    var tracelib = require('./fixtures/trace-lib')
+    function callold() { tracelib.old() }
+    captureStderr(callold).should.containEql(' trace-lib deprecated old\n    at callold (')
+  })
+
+  it('should not trace non-given namespace', function () {
+    var tracelib = require('./fixtures/trace-lib')
+    function callold() { tracelib.old2() }
+    captureStderr(callold).should.containEql(' trace-lib-other deprecated old2 at ')
+  })
+
+  describe('when output supports colors', function () {
+    var stderr
+    before(function () {
+      var tracelib = require('./fixtures/trace-lib')
+      function callold() { tracelib.old() }
+      stderr = captureStderr(callold, true)
+    })
+
+    it('should log in color', function () {
+      stderr.should.not.be.empty
+      stderr.should.containEql('\x1b[')
+    })
+
+    it('should log namespace', function () {
+      stderr.should.containEql('trace-lib')
+    })
+
+    it('should log call site in color', function () {
+      stderr.should.containEql(basename(__filename))
+      stderr.should.match(/\x1b\[\d+mat callold \(/)
+    })
+  })
+})
+
+describe('node script.js', function () {
+  it('should display deprecation message', function (done) {
+    captureChildStderr([script], function (err, stderr) {
+      if (err) return done(err)
+      var filename = path.relative(process.cwd(), script)
+      stderr = stderr.replace(/\w+, \d+ \w+ \d+ \d+:\d+:\d+ \w+/, '__timestamp__')
+      stderr.should.equal('__timestamp__ my-cool-module deprecated oldfunction at ' + filename + ':7:10\n')
+      done()
+    })
+  })
+})
+
+;(function () {
+  // --*-deprecation switches are 0.8+
+  // no good way to feature detect this sync
+  var describe = /^v0\.6\./.test(process.version)
+    ? global.describe.skip
+    : global.describe
+
+  describe('node --no-deprecation script.js', function () {
+    it('should suppress deprecation message', function (done) {
+      captureChildStderr(['--no-deprecation', script], function (err, stderr) {
+        if (err) return done(err)
+        stderr.should.be.empty
+        done()
+      })
+    })
+  })
+
+  describe('node --trace-deprecation script.js', function () {
+    it('should suppress deprecation message', function (done) {
+      captureChildStderr(['--trace-deprecation', script], function (err, stderr) {
+        if (err) return done(err)
+        stderr = stderr.replace(/\w+, \d+ \w+ \d+ \d+:\d+:\d+ \w+/, '__timestamp__')
+        stderr.should.startWith('__timestamp__ my-cool-module deprecated oldfunction\n    at run (' + script + ':7:10)\n    at')
+        done()
+      })
+    })
+  })
+}())
+
+function captureChildStderr(args, callback) {
+  var chunks = []
+  var env = {PATH: process.env.PATH}
+  var exec = process.argv[0]
+  var proc = spawn(exec, args, {
+    env: env
+  })
+
+  proc.stdout.resume()
+  proc.stderr.on('data', function ondata(chunk) {
+    chunks.push(chunk)
+  })
+
+  proc.on('error', callback)
+  proc.on('exit', function () {
+    var stderr = bufferConcat(chunks).toString('utf8')
+    callback(null, stderr)
+  })
+}
+
 function captureStderr(fn, color) {
   var chunks = []
   var isTTY = process.stderr.isTTY
@@ -491,5 +671,5 @@ function captureStderr(fn, color) {
     process.stderr.write = write
   }
 
-  return Buffer.concat(chunks).toString('utf8')
+  return bufferConcat(chunks).toString('utf8')
 }
